@@ -12,7 +12,7 @@
  * The Original Code is the Sablotron XSLT Processor.
  * 
  * The Initial Developer of the Original Code is Ginger Alliance Ltd.
- * Portions created by Ginger Alliance are Copyright (C) 2000-2002
+ * Portions created by Ginger Alliance are Copyright (C) 2000-2003
  * Ginger Alliance Ltd. All Rights Reserved.
  * 
  * Contributor(s): Christian Lefebvre, Stefan Behnel
@@ -100,7 +100,8 @@ typedef enum
     ID_NONE,
     ID_SPS, ID_SPF, ID_SPS_ON_FILES, ID_BASE, ID_TIMES,
     ID_LOG, ID_HELP, ID_DEBUG, ID_DEBUGGER, ID_DBG_HELP, ID_FLAGS, 
-    ID_VERSION, ID_MEASURE, ID_BATCH_XML, ID_BATCH_XSL
+    ID_VERSION, ID_MEASURE, ID_BATCH_XML, ID_BATCH_XSL,
+    ID_CHAIN_XSL
 } SwitchId;
 
 struct SwitchData
@@ -128,6 +129,7 @@ struct SwitchData switches[] =
     {ID_MEASURE, "measure", 'm', 'O', &numberMeasure},
     {ID_BATCH_XML, "batch-xml", 'x', 'O', &batchMode},
     {ID_BATCH_XSL, "batch-xsl", 's', 'O', &batchMode},
+    {ID_CHAIN_XSL, "chain-xsl", 'c', 'O', &batchMode},
     {ID_NONE, NULL, 0, 0, NULL }
 };
 
@@ -139,13 +141,16 @@ char usage[] =
 "\nUsage for single XSLT:\n"\
 "\tsabcmd [options] <stylesheet> [<input> [<output>]] [assignments]\n"\
 "\nUsage in batch processing:\n"\
+"    chain multiple stylesheets, starting with a single input file:\n"\
+"    \tsabcmd [options] --chain-xsl <input> <output> [<stylesheet>]+ [assignments]\n"\
 "    run multiple stylesheets on a single input file:\n"\
 "    \tsabcmd [options] --batch-xml <input> [<stylesheet> <output>]+ [assignments]\n"\
 "    run a single stylesheet on multiple input files:\n"\
 "    \tsabcmd [options] --batch-xsl <stylesheet> [<input> <output>]+ [assignments]\n"\
 "Options:\n"\
-"\t--batch-xml, -x      multiple stylesheets, single input file\n"\
-"\t--batch-xsl, -s      single stylesheet, multiple input files\n"\
+"\t--chain-xsl, -c      single input file, multiple chained stylesheets\n"\
+"\t--batch-xml, -x      single input file, multiple stylesheets\n"\
+"\t--batch-xsl, -s      multiple input files, single stylesheet\n"\
 "\t--base=NAME, -b      set the hard base URI to NAME\n"\
 "\t--debug-options      display information on debugging options\n"\
 "\t--help, -?           display this help message\n"\
@@ -163,7 +168,7 @@ char usage[] =
 char askhelp[] = "Type sabcmd --help to display a help message.\n";
 char msgConflict[] = "conflict with preceding switches: ";
 char version_txt[] = "\nsabcmd "SAB_VERSION" ("SAB_DATE")\n"\
-    "copyright (C) 2000 - 2002 Ginger Alliance (www.gingerall.com)\n";
+    "copyright (C) 2000 - 2003 Ginger Alliance (www.gingerall.com)\n";
 char dbg_usage[] = "\nDebugging options:\n"\
     "\t--debug\t\t\tdisplay results of the command line parse\n"\
     "\t--debugger\t\trun the xslt debugger\n"\
@@ -519,14 +524,18 @@ int runBatchXSLT(const char **arrayParams, const char **arrayArgs,
 		 char **resultArg)
 {
    void *theProcessor = NULL, *sabSit = NULL, *preparsed = NULL;
-   char *resultURI = NULL;
+   char *resultURI = (char*)"file://stdout";
+   char *input = NULL;
    int ecode;
+   int argcount = 2, firstarg = 1;
 
-   if ((batchMode != ID_BATCH_XML) && (batchMode != ID_BATCH_XSL))
+   if ((batchMode != ID_BATCH_XML) &&
+       (batchMode != ID_BATCH_XSL) &&
+       (batchMode != ID_CHAIN_XSL))
       return runSingleXSLT(arrayParams, arrayArgs, resultArg);
 
-
-   if (ecode = SablotCreateSituation(&sabSit))
+   ecode = SablotCreateSituation(&sabSit);
+   if (ecode != 0)
       return ecode;
 
    if (commandFlags)
@@ -542,26 +551,24 @@ int runBatchXSLT(const char **arrayParams, const char **arrayArgs,
       if (stringBase)
 	 SablotSetBase(theProcessor, stringBase);
 
-      ecode = SablotCreateDocument(sabSit, &preparsed);
-
-      if (ecode == 0)
+      switch (batchMode)
       {
-	 if (batchMode == ID_BATCH_XML)
-	    ecode = SablotParse(sabSit, arrayBare[0], &preparsed);
-	 else // if (batchMode == ID_BATCH_XSL)
+	 case ID_BATCH_XSL:
 	    ecode = SablotParseStylesheet(sabSit, arrayBare[0], &preparsed);
+	    break;
+	 case ID_BATCH_XML:
+	    ecode = SablotParse(sabSit, arrayBare[0], &preparsed);
+	    break;
+	 case ID_CHAIN_XSL:
+	    input = arrayBare[0];
+	    argcount = 1;
+	    firstarg = 2;
+	    break;
       }
    }
 
-   if (ecode == 0)
-   {
-      for (int t = 1; (t < indexBare-1) && (ecode == 0); t += 2)
+   for (int t = firstarg; (t <= indexBare-argcount) && (ecode == 0); t += argcount)
       {
-	 if (batchMode == ID_BATCH_XML)
-	    SablotAddArgTree(sabSit, theProcessor, "source", preparsed);
-	 else // if (batchMode == ID_BATCH_XSL)
-	    SablotAddArgTree(sabSit, theProcessor, "stylesheet", preparsed);
-
 	 if (arrayParams)
 	 {
 	    const char **params = arrayParams;
@@ -584,25 +591,51 @@ int runBatchXSLT(const char **arrayParams, const char **arrayArgs,
 
 	 if (ecode == 0)
 	 {
-	    char *output   = arrayBare[t+1];
-
-	    if (batchMode == ID_BATCH_XML)
+	 switch (batchMode)
 	    {
-	       char *stylesheet = arrayBare[t];
-	       ecode = SablotRunProcessorGen(sabSit, theProcessor, stylesheet, "arg:source", output);
-	    }
-	    else // if (batchMode == ID_BATCH_XSL)
+	    case ID_BATCH_XSL:
 	    {
-	       char *document = arrayBare[t];
+	       const char *document = arrayBare[t];
+	       const char *output = arrayBare[t+1];
+	       SablotAddArgTree(sabSit, theProcessor, "stylesheet", preparsed);
 	       ecode = SablotRunProcessorGen(sabSit, theProcessor, "arg:stylesheet", document, output);
-	    }
+	    } break;
+	    case ID_BATCH_XML:
+	    {
+	       const char *stylesheet = arrayBare[t];
+	       const char *output = arrayBare[t+1];
+	       SablotAddArgTree(sabSit, theProcessor, "source", preparsed);
+	       ecode = SablotRunProcessorGen(sabSit, theProcessor, stylesheet, "arg:source", output);
+	    } break;
+	    case ID_CHAIN_XSL:
+	    {
+	       const char *stylesheet = arrayBare[t];
+	       char *output;
+	       if (t < indexBare-argcount)
+		  output = (char*)"arg:output";
+	       else
+		  output = arrayBare[1];
+
+	       ecode = SablotRunProcessorGen(sabSit, theProcessor, stylesheet, input, output);
 
 	    if (ecode == 0)
-	       SablotGetResultArg(theProcessor, resultURI, resultArg);
+	       {
+		  if (input == arrayBare[0])
+		     input = "arg:buffer";
+		  else
+		     SablotFree(*resultArg);
+
+		  SablotGetResultArg(theProcessor, output, resultArg);
+		  SablotAddArgBuffer(sabSit, theProcessor, "buffer", *resultArg);
+	       }
+	    } break;
 	 }
       }
    }
 
+   if ((ecode == 0) && (batchMode != ID_CHAIN_XSL))
+      SablotGetResultArg(theProcessor, resultURI, resultArg);
+
    if (preparsed)
       SablotDestroyDocument(sabSit, preparsed);
 
@@ -686,49 +719,13 @@ int main(int argc, char *argv[])
         {
         case 0: 
 	  {
-	    if ((batchMode == ID_BATCH_XML) || (batchMode == ID_BATCH_XSL))
+	     if ((batchMode == ID_BATCH_XML) || (batchMode == ID_BATCH_XSL) || (batchMode == ID_CHAIN_XSL))
 	      ecode = runBatchXSLT((const char**)arrayParams, 
 		(const char**)arrayArgs, &resultArg);
 	    else
 	      ecode = runSingleXSLT((const char**)arrayParams, 
 				    (const char**)arrayArgs, &resultArg);
 	  }; break;
-	   /*
-            {
-                void *theProcessor;
-                char *resultURI = NULL;
-		void *situa;
-		
-		SablotCreateSituation(&situa);
-		if (commandFlags)
-		  SablotSetOptions(situa, commandFlags);
-
-                if (ecode = SablotCreateProcessorForSituation(situa,
-							      &theProcessor))
-		  break;
-
-                if (stringLog)
-                    SablotSetLog(theProcessor, stringLog, 0);
-
-		if (stringBase)
-		    SablotSetBase(theProcessor, stringBase);
-
-                if (indexBare <= 2)
-                    resultURI = (char*)"file://stdout";
-                else
-                    resultURI = arrayBare[2];
-
-                if (0 != (ecode = SablotRunProcessor(theProcessor,
-                    arrayBare[0],
-                    indexBare <= 1  ? (char*)"file://stdin" : arrayBare[1],
-                    resultURI,
-                    (const char**)arrayParams, (const char**) arrayArgs)))
-                    SablotGetResultArg(theProcessor, resultURI, &resultArg);
-                
-                if (theProcessor)
-                    SablotDestroyProcessor(theProcessor);
-            }; break;
-	   */
         case ID_SPS:
             {
                 if (indexBare != 2)
