@@ -776,7 +776,7 @@ SList<T>::SList(int logBlocksize_)
 template <class T>
 void SList<T>::insert(T what, void *data /* = NULL */)
 {
-    append(what);
+    this->append(what);
     int whatndx = this->number() - 1;
     int i, j;
     for (i=0; i < whatndx; i++)
@@ -793,10 +793,10 @@ template <class T>
 void SList<T>::insertsort(int bottom, int top, void *data)
 {
     int ceiling, k;
-    T curr;
+    //T curr;
     for (ceiling = bottom + 1; ceiling <= top; ceiling++)
     {
-        curr = this->block[ceiling];
+      //curr = this->block[ceiling];
         /* was:
         for (k = ceiling - 1; k >= bottom && compare(k, ceiling) > 0; k--)
             this->block[k+1] = this->block[k];
