@@ -35,6 +35,7 @@
 #include "decimal.h"
 #include "expr.h"
 #include "math.h"
+#include "guard.h"
 
 DecimalFormat::DecimalFormat(const EQName& name_)
     : name(name_),
@@ -131,7 +132,10 @@ eFlag DecimalFormat::format(Sit S, Number& num, const Str& fmt, Str& result)
     
     double theNumber = fabs((double) num * factor);
     
-    char buff[64];
+    //char buff[64];
+    //deallocates automatically
+    GPA( GChar ) buff = new char[1024];
+
     DStr auxFmt = "%";
     if (iDigitsMin)
 	auxFmt += "0";
@@ -140,12 +144,12 @@ eFlag DecimalFormat::format(Sit S, Number& num, const Str& fmt, Str& result)
     auxFmt += ".";
     auxFmt += fDigits;
     auxFmt += "f";
-    sprintf(buff, (char*) auxFmt, theNumber);
+    sprintf((char*)buff, (char*) auxFmt, theNumber);
 
     // buff now holds the number without sign, grouping marks and with possibly too many trailing zeroes
     // scan the number for decimal point position and trailing zeroes count
     //if #.00 was specified, we still have a leading zero (0.12 instead of .12)
-    char *buff1 = buff;
+    char *buff1 = (char*)buff;
     if ( ! iDigitsMin ) //remove leading zeros
       while ( *buff1 == '0' ) buff1++;
 
