@@ -71,6 +71,11 @@ enum OutputterFlags
     { if (mySAXHandler)\
         { mySAXHandler -> FUNC(mySAXUserData, S.getProcessor(), (ARG1), (ARG2)); }}
 
+//macros on internal handler
+#define IF_INT_SAX4( FUNC, ARG1, ARG2, ARG3 )\
+    { if (mySAXHandler)\
+        { ((SAXHandlerInternal*)mySAXHandler) -> FUNC(mySAXUserData, S.getProcessor(), (ARG1), (ARG2), (ARG3)); }}
+
 #define IF_PH1( FUNC )\
     { if (physical)\
         { physical -> FUNC(S); }}
@@ -408,6 +413,7 @@ PhysicalOutputLayerObj::PhysicalOutputLayerObj(CDesc encodingCD_)
     indent = FALSE;
     after_markup = FALSE;  //pc
     level = 0;  //pc
+    defaultNSWas = false;
 }
 
 PhysicalOutputLayerObj::~PhysicalOutputLayerObj()
@@ -440,8 +446,8 @@ eFlag PhysicalOutputLayerObj::outputTrailingNewline (Sit S)
 
 eFlag PhysicalOutputLayerObj::outputDone(Sit S)
 {
-    E( flushBuffer(S) );
-    return OK;
+  E( flushBuffer(S) );
+  return OK;
 }
 
 Str _localName(Str fullname)
@@ -476,9 +482,18 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
     const Str* prefix;
     for (i = namespace_index; i < namespaces.number(); i++)
     {
-      prefix = &(namespaces[i] -> prefix);
-      if (! namespaces.isHidden(*prefix))
+      NamespaceStackObj * nm = namespaces[i];
+      prefix = &(nm -> prefix);
+      //prefix = &(namespaces[i] -> prefix); 
+      //if (!namespaces.isHidden(*prefix))
+
+      // swallow hidden namespaces as well as the empty default
+      //namespace, if no default namespace was output recently
+      if (! nm -> hidden &&
+	  (!(prefix -> isEmpty()) || !(nm -> uri.isEmpty()) || defaultNSWas))
 	{
+	  defaultNSWas = prefix -> isEmpty();
+
           sendLit(S, " xmlns");
           if (!prefix -> isEmpty())
           {
@@ -486,7 +501,7 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
              E( sendStr(S, *prefix) );
           }
           sendLit(S, "=\"");
-          E( sendStrEsc(S, namespaces[i] -> uri, 
+          E( sendStrEsc(S, nm -> uri, 
                         method == OUTPUT_HTML || method == OUTPUT_XHTML ?
                         ESCAPING_HTML_URI : ESCAPING_URI) );
           sendLit(S, "\"");
@@ -540,7 +555,8 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
 	      }
 	    else 
 	      {
-		escapingMode = ESCAPING_HTML_ATTR;
+		if (method == OUTPUT_HTML) 
+		  escapingMode = ESCAPING_HTML_ATTR;
 	      }
 	  }
 
@@ -950,21 +966,22 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
 	    default:
 	        {
 		        // escape non-ASCII values in URIs
-		        if (*data < 0)
-			    {
-                    switch(escapeMode)
-                    {
-                    case ESCAPING_URI:
-                    case ESCAPING_HTML_URI:
-                        {
+		  if (*data & 0x80)
+		    {
+		      switch(escapeMode)
+			{
+			case ESCAPING_URI:
+			case ESCAPING_HTML_URI:
+			  {
                             E( sendOut(S, smallBuf, 
-                                writeCharacterRef(smallBuf, data, escapeMode),
-                                ESCAPING_NONE) );
+				       writeCharacterRef(smallBuf, data, 
+							 escapeMode),
+				       ESCAPING_NONE) );
                             served = TRUE;
-                        }; break;
-                    }
-			    }			        
-		    }
+			  }; break;
+			}
+		    }			        
+		}
         };
 
 
@@ -1005,7 +1022,8 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                 {
                     // unable to convert
                     destCharLen = writeCharacterRef(smallBuf, data, escapeMode);
-                    if (escapeMode == ESCAPING_NONE)
+                    //if (escapeMode == ESCAPING_NONE)
+		    if (method == OUTPUT_TEXT)
                         Err1(S, E1_BAD_CHAR_IN_ENC, smallBuf)
                     else
 		      {
@@ -1122,24 +1140,25 @@ OutputterObj* OutputDocument::setOutputter(OutputterObj* new_)
 
 Str* OutputterObj::nameForSAX(Sit S, const EQName& q)
 {
-  if (q.getUri().isEmpty())
-    return new Str(q.getLocal());
-  DStr temp = q.getUri();
-  temp += THE_NAMESPACE_SEPARATOR;
-  temp += q.getLocal();
-  return new Str(temp);
-
-//    Str aliname;
-//    if (S.getProcessor())
-//      aliname = S.getProcessor() -> getAliasedName(q, currNamespaces);
-//    else
-//      q.getname(aliname);
-//    char *p = new char[aliname.length() + 1];
-//    strcpy(p, (char*) aliname);
-//    char *colon = strchr(p, ':');
-//    if (colon) 
-//      *colon = THE_NAMESPACE_SEPARATOR;
-//    return new Str(p);
+  DStr temp;
+  if (mySAXOutputType == SAXOUTPUT_COPY_TREE || 
+      mySAXOutputType == SAXOUTPUT_INT_PHYSICAL)
+    {
+      if (q.getUri().isEmpty())
+	return new Str(q.getLocal());
+      temp = q.getUri();
+      temp += THE_NAMESPACE_SEPARATOR;
+      temp += q.getLocal();
+      temp += THE_NAMESPACE_SEPARATOR;
+      temp += q.getPrefix();
+      return new Str(temp);
+    }
+  else
+    {
+      Str s;
+      q.getname(s);
+      return new Str(s);
+    }
 };
 
 OutputterObj::OutputterObj()
@@ -1149,15 +1168,12 @@ OutputterObj::OutputterObj()
     physical = NULL;
     outDef = NULL;
     mySAXHandler = NULL;
-    mySAXOutputType = SAXOUTPUT_COPY_TREE;
+    mySAXOutputType = SAXOUTPUT_NONE;
     method = OUTPUT_UNKNOWN;
     noElementYet = TRUE;
     noHeadYet = TRUE;
     delayedDTD = FALSE;
-       // to recognize the first HEAD element
-    //this was an old-fashioned way to supress output of namespaces
-    //currNamespaces.appendConstruct("xsl",Str(theXSLTNamespace),TRUE);
-    //currNamespaces.appendConstruct("xml",Str(theXMLNamespace),TRUE);
+    // to recognize the first HEAD element
 }
 
 OutputterObj::~OutputterObj()
@@ -1326,18 +1342,18 @@ eFlag OutputterObj::eventBeginOutput(Sit S)
 {
   const EQName dummy;
   pushLevel(dummy); //experimental
-    method = outDef ? outDef -> getMethod() : OUTPUT_UNKNOWN;
-    if (physical)
+  method = outDef ? outDef -> getMethod() : OUTPUT_UNKNOWN;
+  if (physical)
     {
       //??JP??both physical & method !=... 
       //duplicit with the same in reportXMLDeclIfMust 
       if (method != OUTPUT_UNKNOWN)
-            E( reportXMLDeclIfMust(S) );
+	E( reportXMLDeclIfMust(S) );
     }
-    // initialize the SAX interface
-    IF_SAX1( startDocument ); 
-    state = STATE_OUTSIDE;
-    return OK;
+  // initialize the SAX interface
+  IF_SAX1( startDocument ); 
+  state = STATE_OUTSIDE;
+  return OK;
 }
 
 eFlag OutputterObj::eventBeginSubtree(Sit S)
@@ -1358,52 +1374,52 @@ eFlag OutputterObj::eventBeginSubtree(Sit S)
 
 eFlag OutputterObj::eventElementStart(Sit S, const EQName& name)
 {
-    if (noElementYet)
+  if (noElementYet)
     {
-        noElementYet = FALSE;
-        if (physical)
+      noElementYet = FALSE;
+      if (physical)
         {
-            if (method == OUTPUT_UNKNOWN)
+	  if (method == OUTPUT_UNKNOWN)
             {
-                if (name.getUri().isEmpty() && name.getLocal().eqNoCase("html"))
-                    method = OUTPUT_HTML;
-                else
-                    method = OUTPUT_XML;
-                E( physical -> setMethodByDefault(S, method) );
-                E( reportXMLDeclIfMust(S) );
-                E( reportFront(S) );//??JP??sax
+	      if (name.getUri().isEmpty() && name.getLocal().eqNoCase("html"))
+		method = OUTPUT_HTML;
+	      else
+		method = OUTPUT_XML;
+	      E( physical -> setMethodByDefault(S, method) );
+	      E( reportXMLDeclIfMust(S) );
+	      E( reportFront(S) );//??JP??sax
             };
-            // If this is an included stylesheet, output method cd be TEXT here
-            if (method != OUTPUT_TEXT) {
-	        E( reportDTDIfMust( S, name ) );//??JP??sax
-            }
+	  // If this is an included stylesheet, output method cd be TEXT here
+	  if (method != OUTPUT_TEXT) {
+	    E( reportDTDIfMust( S, name ) );//??JP??sax
+	  }
         };
     }
-    switch(state)
+  switch(state)
     {
     case STATE_OUTSIDE:
     case STATE_IN_MARKUP:
     case STATE_IN_ELEMENT:
-        {
-            // reportStartTag also sends the event to SAX
-            E( reportStartTag(S, NONEMPTY_ELEMENT) );
-            E( reportCurrData(S) );
-            pushLevel(name);
-            // this sets state to STATE_IN_MARKUP
-        };
-        break;
+      {
+	// reportStartTag also sends the event to SAX
+	E( reportStartTag(S, NONEMPTY_ELEMENT) );
+	E( reportCurrData(S) );
+	pushLevel(name);
+	// this sets state to STATE_IN_MARKUP
+      };
+      break;
     case STATE_IN_COMMENT:
     case STATE_IN_PI:
     case STATE_IN_ATTRIBUTE:
-        Err( S, E_ELEM_IN_COMMENT_PI );
+      Err( S, E_ELEM_IN_COMMENT_PI );
     default:
-        assert(!"eventElementStart");
+      assert(!"eventElementStart");
     };
-
-    // determine whether we're in a HEAD html element so a META tag
-    // needs to be added
-
-    return OK;
+  
+  // determine whether we're in a HEAD html element so a META tag
+  // needs to be added
+  
+  return OK;
 }
 
 eFlag OutputterObj::throwInMeta(Sit S)
@@ -1461,71 +1477,84 @@ eFlag OutputterObj::throwInMeta(Sit S)
 
 eFlag OutputterObj::eventElementEnd(Sit S, const EQName& name)
 {
-    Str temp;
-    if (S.getProcessor())
-      temp = S.getProcessor() -> getAliasedName(name, 
-					      currNamespaces);
-    else name.getname(temp);
-
-    switch(state)
+  assert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
+  Str physName;
+  //  if (S.getProcessor())
+  //  temp = S.getProcessor() -> getAliasedName(name, 
+  //					      currNamespaces,
+  //					      mySAXOutputType == SAXOUTPUT_INT_PHYSICAL);
+  //else name.getname(temp);
+  
+  switch(state)
     {
     case STATE_IN_MARKUP:
-        E( reportStartTag(S, EMPTY_ELEMENT) ); 
-        break;
+      E( reportStartTag(S, EMPTY_ELEMENT) ); 
+      break;
     case STATE_IN_ELEMENT:
-        {
-            E( reportCurrData(S) );
-            if (physical)
-            {
-                physical -> outputElementEnd(S, temp, NONEMPTY_ELEMENT );
-            };
-        }; break;
+      {
+	E( reportCurrData(S) );
+	if (physical)
+	  {
+	    Str physName;
+	    name.getname(physName);
+	    physical -> outputElementEnd(S, physName, NONEMPTY_ELEMENT );
+	  };
+      }; break;
     default:
-        assert(!"eventElementEnd");
+      assert(!"eventElementEnd");
     };
-
-    // send the event to SAX
-    switch(mySAXOutputType)
+  
+  // send the event to SAX
+  switch(mySAXOutputType)
     {
     case SAXOUTPUT_COPY_TREE:
-        {
+    case SAXOUTPUT_INT_PHYSICAL:
+      {
         GP( Str ) theSAXname = nameForSAX(S, name);
 	IF_SAX2( endElement, (const char*) *theSAXname );
 	theSAXname.del();
 	// report namespace scope ends
-	while (currNamespaces.number() > getFirstOwnNS())
+	//while (currNamespaces.number() > getFirstOwnNS())
+	for (int ii = currNamespaces.number() - 1; ii >= getFirstOwnNS(); ii--)
 	  {
-	    IF_SAX2( endNamespace, currNamespaces.last() -> prefix );
-	    currNamespaces.freelast(FALSE);
+	    IF_SAX2( endNamespace, currNamespaces[ii] -> prefix );
+	    //debug
 	  }
-	};
-	break;
+      };
+      break;
     case SAXOUTPUT_AS_PHYSICAL:
-        {
-        IF_SAX2( endElement, (const char*) temp );
+      {
+	Str physName;
+	name.getname(physName);
+        IF_SAX2( endElement, (const char*) physName );
 	// report namespace scope ends
 	Str* prefix;
-	while (currNamespaces.number() > getFirstOwnNS())
+	//while (currNamespaces.number() > getFirstOwnNS())
+	for (int ii = currNamespaces.number() - 1; ii >= getFirstOwnNS(); ii--)
 	  {
-	    prefix = &(currNamespaces.last() -> prefix);
+	    prefix = &(currNamespaces[ii] -> prefix);
 	    if (! currNamespaces.isHidden(*prefix))
-	      IF_SAX2( endNamespace, currNamespaces.last() -> prefix );
-	    currNamespaces.freelast(FALSE);
+	      IF_SAX2( endNamespace, currNamespaces[ii] -> prefix );
 	  }
-	};
-        break;
+      };
+      break;
+    case SAXOUTPUT_NONE: break;
     default:
-        assert(!"eventElementEnd");
+      assert(!"eventElementEnd");
     };
+  
+  //remove our namespaces
+  while (currNamespaces.number() > getFirstOwnNS())
+    currNamespaces.freelast(FALSE);
 
-    // pop one history level
-    history.freelast(FALSE);
-    if (getLevel()) //experimental
-        state = STATE_IN_ELEMENT;
-    else
-        state = STATE_OUTSIDE;
-
-    return OK;
+  // pop one history level
+  history.freelast(FALSE);
+  if (getLevel()) //experimental
+    state = STATE_IN_ELEMENT;
+  else
+    state = STATE_OUTSIDE;
+  
+  return OK;
 }
 
 
@@ -1663,6 +1692,7 @@ eFlag OutputterObj::eventNamespace(Sit S, const Str& prefix, const Str& uri,
     //xsl:element insrtuction) in this case (existing_ndx >=
     //getFirstOwnNS()) we need override former definition in place.
 
+    /*
     if (uri == theXSLTNamespace)
       {
 	//we need to reveal the xslt namespace if it is aliased
@@ -1674,9 +1704,13 @@ eFlag OutputterObj::eventNamespace(Sit S, const Str& prefix, const Str& uri,
 	    hidden = hidden && ! aliased;
 	  }
       }
+    */
 
     if (!existing_namespace)
-      currNamespaces.appendConstruct(prefix, uri, hidden);
+      {
+	//if the namespace doesn't exist yet, we add it
+	currNamespaces.appendConstruct(prefix, uri, hidden);
+      }
     else
       {
 	Bool exIsHidden = currNamespaces[existing_ndx]->hidden;
@@ -1686,6 +1720,7 @@ eFlag OutputterObj::eventNamespace(Sit S, const Str& prefix, const Str& uri,
 	    if (existing_ndx >= getFirstOwnNS())
 	      {
 		currNamespaces[existing_ndx] -> uri = uri;
+		currNamespaces[existing_ndx] -> hidden = hidden;
 	      }
 	    else
 	      {
@@ -1711,9 +1746,9 @@ eFlag OutputterObj::eventDisableEscapingForNext(Sit S)
         case STATE_IN_ATTRIBUTE:
         case STATE_IN_PI:
         case STATE_IN_COMMENT:
-            Warn(S, W_DISABLE_OUTPUT_ESC);
+	  Warn(S, W_DISABLE_OUTPUT_ESC); break;
         default:
-            outputEscaping = FALSE;
+	  outputEscaping = FALSE;
         };
     }
     return OK;
@@ -1855,131 +1890,151 @@ eFlag OutputterObj::reportCurrData(Sit S, Bool hardCData /* = FALSE */)
 
 eFlag OutputterObj::reportStartTag(Sit S, Bool isEmpty)
 {
-    // FIXME: couldn't improve reportStartTag from GP point of view?
-    if (state == STATE_OUTSIDE || currElement.isEmpty())
-        return OK;
+  assert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
+  // FIXME: couldn't improve reportStartTag from GP point of view?
+  if (state == STATE_OUTSIDE || currElement.isEmpty())
+    return OK;
+  
+  int doThrowMeta = method == OUTPUT_HTML && noHeadYet && 
+    currElement.getUri().isEmpty() && currElement.getLocal().eqNoCase("head");
+  int i;
+  //get attr aliased names
+  StrStrList attAliased;
+  // the temporary string is a workaround for VisualAge on AIX
+  //Str temp;
 
-    int doThrowMeta = method == OUTPUT_HTML && noHeadYet && 
-      currElement.getUri().isEmpty() && currElement.getLocal().eqNoCase("head");
-    int i;
-    //get attr aliased names
-    StrStrList attAliased;
-    // the temporary string is a workaround for VisualAge on AIX
-    Str temp;
 
-    if (physical || (mySAXHandler && mySAXOutputType == SAXOUTPUT_AS_PHYSICAL))
-      {
-	if (S.getProcessor())
-	  temp = S.getProcessor() -> getAliasedName(currElement, currNamespaces);
-	else
-	  currElement.getname(temp);
+  Str physName;
+  Bool intPhys = mySAXOutputType == SAXOUTPUT_INT_PHYSICAL;
 
-	for (i = 0; i < currAtts.number(); i++)
+  if (physical || (mySAXHandler && (mySAXOutputType == SAXOUTPUT_AS_PHYSICAL)))
+    {
+      //if (S.getProcessor())
+      //temp = S.getProcessor() -> getAliasedName(currElement, 
+      //					  currNamespaces, intPhys);
+      //else
+      //currElement.getname(temp);
+      currElement.getname(physName);
+      
+      for (i = 0; i < currAtts.number(); i++)
+	{
+	  Str aliname;
+	  //if (S.getProcessor())
+	  //  aliname = S.getProcessor() -> getAliasedName(currAtts[i] -> key,
+	  //					 currNamespaces, 
+	  //					 intPhys);
+	  //else
+	  //  currAtts[i] -> key.getname(aliname); 
+	  currAtts[i] -> key.getname(aliname);
+	  attAliased.appendConstruct(aliname, currAtts[i] -> value);
+	}
+      
+      if (physical) {
+	if ( delayedDTD )
 	  {
-	    Str aliname;
-	    if (S.getProcessor())
-	      aliname = S.getProcessor() -> getAliasedName(currAtts[i] -> key,
-							   currNamespaces);
-	    else
-	      currAtts[i] -> key.getname(aliname); 
-	    attAliased.appendConstruct(aliname, currAtts[i] -> value);
+	    const Str& 
+	      DTSystem = outDef -> getValueStr(XSLA_DOCTYPE_SYSTEM),
+	      DTPublic = outDef -> getValueStr(XSLA_DOCTYPE_PUBLIC);        
+	    E( physical -> outputDTD( S, physName, DTPublic, DTSystem) );
+	    delayedDTD = FALSE;
 	  }
 
-	if (physical) {
-	  if ( delayedDTD )
-	    {
-	      const Str& 
-		DTSystem = outDef -> getValueStr(XSLA_DOCTYPE_SYSTEM),
-		DTPublic = outDef -> getValueStr(XSLA_DOCTYPE_PUBLIC);        
-	      E( physical -> outputDTD( S, temp, DTPublic, DTSystem) );
-	      delayedDTD = FALSE;
-	    }
-	  
-	  E( physical -> outputElementStart(S,
-					    temp, currNamespaces, 
-					    getFirstOwnNS(), attAliased, 
-					    isEmpty && !doThrowMeta) );
-	}
+	E( physical -> outputElementStart(S, physName, currNamespaces, 
+					  getFirstOwnNS(), attAliased, 
+					  isEmpty && !doThrowMeta) );
       }
-
-    if (mySAXHandler)
+    }
+  
+  if (mySAXHandler)
     {
       const char** attsTable;
-        GPA( ConstCharP ) attsArr = attsTable = new const char*[(currAtts.number() * 2) + 1];
-        // GP: the allocations are OK: no exit routes
-
-        PList<Str*> stringsForDeletion;
-        attsTable[currAtts.number() * 2] = NULL;
-
-	switch(mySAXOutputType)
+      GPA( ConstCharP ) attsArr = attsTable = new const char*[(currAtts.number() * 2) + 1];
+      // GP: the allocations are OK: no exit routes
+      
+      PList<Str*> stringsForDeletion;
+      attsTable[currAtts.number() * 2] = NULL;
+      
+      switch(mySAXOutputType)
+	{
+	case SAXOUTPUT_COPY_TREE:
+	case SAXOUTPUT_INT_PHYSICAL:
 	  {
-	  case SAXOUTPUT_COPY_TREE:
-	    {
-	      // report only the new namespaces
-	      for (i = getFirstOwnNS(); i < currNamespaces.number(); i++)
-		{
-		  IF_SAX3( startNamespace, currNamespaces[i] -> prefix, 
-			   currNamespaces[i] -> uri );
-		}
-	      // create the attribute table
-	      for (i = 0; i < currAtts.number(); i++)
-		{
-		  Str *expatName = nameForSAX(S, currAtts[i] -> key);
-		  stringsForDeletion.append(expatName);
-		  attsTable[i * 2] = *expatName;
-		  attsTable[(i * 2) + 1] = currAtts[i] -> value;
-		};
-	      Str* theSAXname = nameForSAX(S, currElement);
-	      stringsForDeletion.append(theSAXname);
-	      IF_SAX3(startElement, *theSAXname, attsTable);
-	      stringsForDeletion.freeall(FALSE);
-	    };
-	    break;
-	  case SAXOUTPUT_AS_PHYSICAL:
-	    // report only the new not hidden namespaces
+	    // report only the new namespaces
 	    Str* prefix;
 	    for (i = getFirstOwnNS(); i < currNamespaces.number(); i++)
 	      {
 		prefix = &(currNamespaces[i] -> prefix);
-		if (! currNamespaces.isHidden(*prefix)) 
+		if (intPhys)
+		  {
+		    IF_INT_SAX4( startNamespace2, currNamespaces[i] -> prefix, 
+				 currNamespaces[i] -> uri, 
+				 currNamespaces.isHidden(*prefix) );
+		  }
+		else
 		  IF_SAX3( startNamespace, currNamespaces[i] -> prefix, 
 			   currNamespaces[i] -> uri );
 	      }
 	    // create the attribute table
 	    for (i = 0; i < currAtts.number(); i++)
 	      {
-		attsTable[i * 2] = (char *)attAliased[i] -> key;
-		attsTable[(i * 2) + 1] = (char *)attAliased[i] -> value;
+		Str *expatName = nameForSAX(S, currAtts[i] -> key);
+		stringsForDeletion.append(expatName);
+		attsTable[i * 2] = *expatName;
+		attsTable[(i * 2) + 1] = currAtts[i] -> value;
 	      };
-	    
-	    IF_SAX3(startElement, temp, attsTable);
-	    break;
-	  default:
-	    assert(!"eventElementEnd");
-	  };
-	//attsTable.delArray();	
-    }
-
-    if (physical || (mySAXHandler && mySAXOutputType == SAXOUTPUT_AS_PHYSICAL))
-      {
-	attAliased.freeall(FALSE);
-      }
-
-    eFlag result = OK;
-    if (doThrowMeta) {
-        result = throwInMeta(S);
-	//__PH__ close head if needed
-	if (isEmpty) {
-	  if (physical) {
-	    E( physical ->outputElementEnd(S, temp, FALSE) );
+	    Str* theSAXname = nameForSAX(S, currElement);
+	    stringsForDeletion.append(theSAXname);
+	    IF_SAX3(startElement, *theSAXname, attsTable);
+	    stringsForDeletion.freeall(FALSE);
 	  };
+	  break;
+	case SAXOUTPUT_AS_PHYSICAL:
+	  // report only the new not hidden namespaces
+	  Str* prefix;
+	  for (i = getFirstOwnNS(); i < currNamespaces.number(); i++)
+	    {
+	      prefix = &(currNamespaces[i] -> prefix);
+	      if (! currNamespaces.isHidden(*prefix)) 
+		IF_SAX3( startNamespace, currNamespaces[i] -> prefix, 
+			 currNamespaces[i] -> uri );
+	    }
+	  // create the attribute table
+	  for (i = 0; i < currAtts.number(); i++)
+	    {
+	      attsTable[i * 2] = (char *)attAliased[i] -> key;
+	      attsTable[(i * 2) + 1] = (char *)attAliased[i] -> value;
+	    };
+	  
+	  IF_SAX3(startElement, physName, attsTable);
+	  break;
+	case SAXOUTPUT_NONE:
+	  assert(!"saxoutput == NONE");
+	default:
+	  assert(!"eventElementEnd");
 	};
+      //attsTable.delArray();	
     }
-
-    currElement.empty();
-    currAtts.freeall(FALSE);
-    currData.empty();
-    return result;
+  
+  if (physical || (mySAXHandler && mySAXOutputType == SAXOUTPUT_AS_PHYSICAL))
+    {
+      attAliased.freeall(FALSE);
+    }
+  
+  eFlag result = OK;
+  if (doThrowMeta) {
+    result = throwInMeta(S);
+    //__PH__ close head if needed
+    if (isEmpty) {
+      if (physical) {
+	E( physical ->outputElementEnd(S, physName, FALSE) );
+      };
+    };
+  }
+  
+  currElement.empty();
+  currAtts.freeall(FALSE);
+  currData.empty();
+  return result;
 }
 
 void OutputterObj::pushLevel(const EQName& name)
