@@ -50,7 +50,7 @@
 //
 //
 
-SAXHandler TreeConstructer::myHandlerRecord =
+SAXHandlerInternal TreeConstructer::myHandlerRecord =
 {
     saxStartDocument,
     saxStartElement,
@@ -60,7 +60,8 @@ SAXHandler TreeConstructer::myHandlerRecord =
     saxComment,
     saxPI,
     saxCharacters,
-    saxEndDocument
+    saxEndDocument,
+    saxStartNamespace2
 };
 
 TreeConstructer::TreeConstructer(Sit S_)
@@ -132,7 +133,7 @@ eFlag TreeConstructer::parseDataLineUsingExpat(Sit S,
 
     XML_SetUserData(theParser, this);
     if (S.getProcessor())
-      XML_SetBase(theParser, S.getProcessor() -> findBaseURI( t ->getURI()));
+      XML_SetBase(theParser, S.getProcessor() -> findBaseURI(S, t ->getURI()));
     else if (base_)
       XML_SetBase(theParser, base_);
     
@@ -152,12 +153,13 @@ eFlag TreeConstructer::parseDataLineUsingExpat(Sit S,
     return OK;
 };
 
-eFlag TreeConstructer::parseUsingSAX(Sit S, Tree *t, OutputterObj& source)
+eFlag TreeConstructer::parseUsingSAX(Sit S, Tree *t, OutputterObj& source,
+				     SAXOutputType ot)
 {
     theTree = t;
     theDataLine = NULL;
     // register the handler with the outputter
-    E( source.setOptionsSAX(S, &myHandlerRecord, this, SAXOUTPUT_COPY_TREE) );
+    E( source.setOptionsSAX(S, (SAXHandler*)&myHandlerRecord, this, ot) );
     E( source.eventBeginOutput(S) );
     return OK;
 }
@@ -178,7 +180,7 @@ eFlag TreeConstructer::parseUsingSAXForAWhile(Sit S, OutputterObj& source,
       E( theTree -> pushPendingNS(S, srcTree, swallowNS) );
 
     // register the handler with the outputter
-    E( source.setOptionsSAX(S, &myHandlerRecord, this, SAXOUTPUT_COPY_TREE) );
+    E( source.setOptionsSAX(S, (SAXHandler*)&myHandlerRecord, this, SAXOUTPUT_COPY_TREE) );
     E( source.eventBeginOutput(S) );
     return OK;
 }
@@ -428,9 +430,9 @@ eFlag TreeConstructer::feedDocumentToParser(Sit S, void* constructer)
         if (!(firstTime && rawBuffer[0] == (char) 0xEF && 
             rawBuffer[1] == (char) 0xBB && rawBuffer[2] == (char) 0xBF))
             S.setCurrLine(XML_GetCurrentLineNumber(parser));
-        int code = XML_GetErrorCode(parser); 
+        XML_Error code = XML_GetErrorCode(parser); 
         Str eCodeStr, eNameStr;
-        eCodeStr = code;
+        eCodeStr = (int)code;
         eNameStr = (char*) XML_ErrorString(code);
         // XML_ParserFree(parser); -- done later
         Err2T(this_, S, E_XML, eCodeStr, eNameStr);
@@ -604,6 +606,11 @@ void TreeConstructer::tcStartElement(
     v -> lineno = elemLine;
     t -> appendVertex(this_ -> S, v);
 
+    //if we are looking at the document element we need to check,
+    // whether the default namespace was declared
+    if (isRoot(v -> parent) && t -> pendingNS().findNdx(UNDEF_PHRASE) == -1)
+      tcStartNamespace(constructer, NULL, ""); //empty default namespace
+
     t -> pendingNS().giveCurrent(this_ -> S, toE(v) -> namespaces, t,
 				 this_ -> nsCount.last());
 #ifndef EXPAT_SUPPORTS_TRIPLETS
@@ -699,7 +706,7 @@ void TreeConstructer::tcEndElement(
     } 
     else if ( isExtension(v) ) 
       {
-        if (toX(v) -> checkChildren(this_ -> S))
+        if (toExtension(v) -> checkChildren(this_ -> S))
             return;
       }
 
@@ -722,6 +729,12 @@ void TreeConstructer::tcEndElement(
 /* static */
 void TreeConstructer::tcStartNamespace(
     void* constructer, const char* prefix, const char* uri)
+{
+  tcStartNamespace2(constructer, prefix, uri, false);
+}
+
+void TreeConstructer::tcStartNamespace2(
+    void* constructer, const char* prefix, const char* uri, Bool hidden)
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
@@ -746,7 +759,9 @@ void TreeConstructer::tcStartNamespace(
 
     // t -> pendingNS().appendAndSetOrdinal(
     t -> pendingNS().append(
-        newv = new(&(t -> getArena())) NmSpace(*t, prefixPh, uriPh));
+        newv = new(&(t -> getArena())) 
+	    NmSpace(*t, prefixPh, uriPh, hidden));
+    
     newv -> lineno = this_ -> getCurrentLineNumber();
     // count of pending namespaces increased
     this_ -> nsCount[this_ -> nsCount.number()-1]++;
@@ -762,8 +777,6 @@ void TreeConstructer::tcStartNamespace(
       }
 };
 
-
-
 //
 //  tcEndNamespace
 //  callback for the namespace scope end event
@@ -967,7 +980,7 @@ int TreeConstructer::tcExternalEntityRef(
         return 0;
 
     Str absolute;
-    makeAbsoluteURI(systemId, base, absolute);
+    makeAbsoluteURI(this_ -> S,systemId, base, absolute);
     XML_SetBase(newParser, absolute); //Christian + PH
 
     GP(DataLine) newDL = new DataLine;
@@ -995,7 +1008,7 @@ int TreeConstructer::tcExternalEntityRef(
     if (!code)
     {
 	code = newTC -> parseDataLineUsingGivenExpat(this_ -> S, t, newDL, newParser);
-	t -> endSubtree(this_ -> S); 
+	t -> endSubtree(this_ -> S, XSL_NONE); 
     }
     (*newDL).close(this_ -> S);
     XML_ParserFree(newParser);
@@ -1014,7 +1027,7 @@ void TreeConstructer::tcEntityDecl(void *userData, const char *entityName,
     {
       Str name = entityName;
       Str absolute;
-      makeAbsoluteURI(systemId, base, absolute);
+      makeAbsoluteURI(this_ -> S, systemId, base, absolute);
       this_ -> theTree -> setUnparsedEntityUri( name, absolute );
     }
 }
@@ -1047,6 +1060,14 @@ void TreeConstructer::saxStartNamespace(void* userData,
   tcStartNamespace(userData, prefix, uri);
 }
 
+void TreeConstructer::saxStartNamespace2(void* userData, 
+					 SablotHandle processor_,
+					 const char* prefix, const char* uri,
+					 Bool hidden)
+{
+  tcStartNamespace2(userData, prefix, uri, hidden);
+}
+
 void TreeConstructer::saxEndNamespace(void* constructer, 
 				      SablotHandle processor_, 
 				      const char* prefix)
@@ -1081,7 +1102,7 @@ void TreeConstructer::saxEndDocument(void* constructer,
   tcEndDocument(constructer);
 }
 
-/* static */
+/* static MUST NOT RELY ON EXPAT_SUPPORTS_TRIPLETS*/
 eFlag TreeConstructer::setQNameFromExpat(Sit S, TreeConstructer* this_, QName& qname_, const char* text)
 {
     Tree *t = this_ -> theTree;
