@@ -309,9 +309,10 @@ eFlag Processor::open(Sit S, const char *sheetURI, const char *inputURI)
     DStr theBase;
 
     my_getcwd(theBase);
-    theBase = findBaseURI(Str("file://") + theBase);
+    theBase = findBaseURI(S, Str("file://") + theBase);
 
     E( readTreeFromURI(S, styleSheet, temp = sheetURI, theBase, TRUE) );
+    //styleSheet -> speakDebug();
 
 #ifdef SABLOT_DEBUGGER
     if (debugger) debugger -> setBreakpoints(S, styleSheet);
@@ -330,7 +331,7 @@ eFlag Processor::run(Sit S, const char* resultURI, NodeHandle doc /*= NULL */)
     Str temp;
     DStr theBase;
     my_getcwd(theBase);
-    theBase = findBaseURI(Str("file://") + theBase);
+    theBase = findBaseURI(S, Str("file://") + theBase);
 
     //strip spaces from the document
     if (input) E( stripTree(S, *input) )
@@ -454,10 +455,12 @@ eFlag Processor::execApplyTemplates(Sit S, Context *c, Bool resolvingGlobals)
     return OK;
 }
 
-eFlag Processor::execApplyImports(Sit S, Context *c, Bool resolvingGlobals)
+eFlag Processor::execApplyImports(Sit S, Context *c, SubtreeInfo *subtree,
+				  Bool resolvingGlobals)
 {
     XSLElement *rule;
-    E( NZ(styleSheet) -> findBestRule(S, rule, c, getCurrentMode(), TRUE) );
+    E( NZ(styleSheet) -> 
+       findBestRule(S, rule, c, getCurrentMode(), TRUE, subtree) );
     if (rule)
         E( rule -> execute(S, c, FALSE) );
     return OK;
@@ -600,10 +603,10 @@ eFlag Processor::addLineParse(Sit S, Tree *& newTree, Str &absolute,
     return OK;
 }
 
-const Str& Processor::findBaseURI(const Str& unmappedBase)
+const Str& Processor::findBaseURI(Sit S, const Str& unmappedBase)
 {
     Str scheme, rest;
-    uri2SchemePath(unmappedBase, scheme, rest);
+    uri2SchemePath(S, unmappedBase, scheme, rest);
     Str *mapping = baseURIMappings.find(scheme);
     if (mapping) return *mapping;
     mapping = baseURIMappings.find(""/**theEmptyString*/);
@@ -617,9 +620,9 @@ const Str& Processor::findBaseURI(const Str& unmappedBase)
 //  returns the *translated* base URI in effect for a given vertex
 //
 
-const Str& Processor::baseForVertex(Element *v)
+const Str& Processor::baseForVertex(Sit S, Element *v)
 {
-    return findBaseURI(NZ(v) -> getSubtreeInfo() -> getBaseURI());
+    return findBaseURI(S, NZ(v) -> getSubtreeInfo() -> getBaseURI());
 }
 
 
@@ -637,7 +640,7 @@ eFlag Processor::readTreeFromURI(Sit S, Tree*& newTree, const Str& location,
 {
     Str
         absolute;
-    makeAbsoluteURI(location, base, absolute);
+    makeAbsoluteURI(S, location, base, absolute);
     newTree = datalines.getTree(absolute, isXSL, DLMODE_READ);
     if (!newTree)
         E( addLineParse(S, newTree, absolute, isXSL, ignoreErr) );
@@ -649,7 +652,7 @@ eFlag Processor::createOutputterForURI(Sit S, Str& location, Str& base,
 				       OutputDefinition *outDef /*=null*/)
 {
   Str absolute;
-  makeAbsoluteURI(location, base, absolute);
+  makeAbsoluteURI(S, location, base, absolute);
   if (datalines.getTree(absolute, FALSE, DLMODE_WRITE))
     Err1(S, E1_URI_WRITE, absolute);
   GP( DataLine ) d;
@@ -766,7 +769,8 @@ eFlag Processor::pushOutputterForURI(Sit S, Str& location, Str& base,
   return OK;
 }
 
-eFlag Processor::pushTreeConstructer(Sit S, TreeConstructer *& newTC, Tree *t)
+eFlag Processor::pushTreeConstructer(Sit S, TreeConstructer *& newTC, Tree *t,
+				     SAXOutputType ot)
 {
     newTC = NULL;
     GP( TreeConstructer ) newTC_ = new TreeConstructer(S);
@@ -774,7 +778,7 @@ eFlag Processor::pushTreeConstructer(Sit S, TreeConstructer *& newTC, Tree *t)
     //E( (*newTCSource).setOptions(S, NULL, &(styleSheet -> outputDef)) ); //_PH_
     M( S, newTC_+0 );
     outputters_.append(newTCSource);
-    E( (*newTC_).parseUsingSAX(S, t, *newTCSource) );
+    E( (*newTC_).parseUsingSAX(S, t, *newTCSource, ot) );
     newTCSource.keep();
     newTC = newTC_.keep();
     return OK;
@@ -964,12 +968,12 @@ RETURNS
                 malloc())
 *****************************************************************/
 
-void Processor::copyArg(const Str& argName, int* argOrdinal,
+void Processor::copyArg(Sit S, const Str& argName, int* argOrdinal,
     char*& newCopy)
 {
     Str absolute;
     int lineNo;
-    if ((makeAbsoluteURI((Str&)argName, "arg:/", absolute) != URI_ARG)
+    if ((makeAbsoluteURI(S, (Str&)argName, "arg:/", absolute) != URI_ARG)
         || (lineNo = datalines.findNum(absolute, FALSE, DLMODE_WRITE)) == -1)
     {
         newCopy = NULL;
@@ -1017,7 +1021,7 @@ eFlag Processor::useTree(Sit S, const char *name, Tree *t)
 	
 	Str absolute;
 	DataLine *d;
-	makeAbsoluteURI(nameStr, "arg:", absolute);
+	makeAbsoluteURI(S, nameStr, "arg:", absolute);
 	E( addLineTreeOnly(S, d, absolute, t -> XSLTree, t) );
     addedFlag = TRUE;
     return OK;
@@ -1076,7 +1080,7 @@ eFlag Processor::freeResultArgs(Sit S)
     return OK;
 }
 
-Arena* Processor::getArena() 
+SabArena* Processor::getArena() 
 {
     return &theArena;
 }
@@ -1100,9 +1104,33 @@ eFlag Processor::prefixIsAliasTarget(Sit S, const Str& prefix, Bool& result)
   return OK;
 }
 
-Str Processor::getAliasedName(const EQName& name, NamespaceStack& currNamespaces)
+void Processor::getAliasedName(EQName & name, Bool & aliased)
+{
+  Str myUri = name.getUri();
+  for (int i = 0; i < styleSheet -> aliases().number(); i++)
+    {
+      const Str & aliasUri = 
+	styleSheet -> expand(styleSheet -> aliases()[i]->getKey());
+      if (aliasUri && aliasUri == myUri)
+	{
+	  Phrase newUri = styleSheet -> aliases()[i] -> getValue();
+	  Phrase newPrefix = styleSheet -> aliases()[i] -> getPrefix();
+	  name.setUri(styleSheet -> expand(newUri));
+	  //name.setPrefix(styleSheet -> expand(newPrefix));
+	  aliased = true;
+	  break;
+	};	        
+    }
+}
+
+/*
+Str Processor::getAliasedName(const EQName& name, 
+			      NamespaceStack& currNamespaces,
+			      Bool expatLike)
 {
    DStr s;
+   Str newPrefixStr;
+   Str newUri;
    //Bool defaultNS = !name.hasPrefix();
 
    if (1 || name.hasPrefix())
@@ -1117,7 +1145,7 @@ Str Processor::getAliasedName(const EQName& name, NamespaceStack& currNamespaces
           {
 	    const Str* aliasUri = 
 	      currNamespaces.getUri(styleSheet -> expand(
-					 styleSheet -> aliases()[i]->getKey()));
+	                         styleSheet -> aliases()[i]->getKey()));
 	    if (aliasUri && *aliasUri == myUri)
               {
 		newPrefix = styleSheet -> aliases()[i] -> getValue();
@@ -1127,24 +1155,39 @@ Str Processor::getAliasedName(const EQName& name, NamespaceStack& currNamespaces
       };
       
       if (newPrefix == UNDEF_PHRASE)
-      {
-        if (! aliasFound && name.hasPrefix()) s += name.getPrefix();
-      } 
+	{
+	  if (! aliasFound && name.hasPrefix()) 
+	    newPrefixStr = name.getPrefix();
+	} 
       else 
-      {
-        Str newPrefixStr = styleSheet -> expand(newPrefix);
-        s += newPrefixStr;
-	//we should proceed w/o this 
-        //currNamespaces.appendConstruct(name.getPrefix(),name.getUri(),TRUE);
-	//currNamespaces.appendConstruct(newPrefixStr,
-        //*currNamespaces.getUri(newPrefixStr));
-      };
-      if (!s.isEmpty()) // could be empty e.g. for the default namespace
-        s += ":";
+	{
+	  newPrefixStr = styleSheet -> expand(newPrefix);
+	  //s += newPrefixStr;
+	  //we should proceed w/o this 
+	  //currNamespaces.appendConstruct(name.getPrefix(),name.getUri(),TRUE);
+	  //currNamespaces.appendConstruct(newPrefixStr,
+	  //*currNamespaces.getUri(newPrefixStr));
+	};
+      //if (!s.isEmpty()) // could be empty e.g. for the default namespace
+      //  s += ":";
    };
-   s += name.getLocal();
+   if (expatLike)
+     {
+       s  = name.getUri();
+       s += THE_NAMESPACE_SEPARATOR;
+       s += name.getLocal();
+       s += THE_NAMESPACE_SEPARATOR;
+       s += newPrefixStr;
+     }
+   else 
+     {
+       s = newPrefixStr;
+       if (! s.isEmpty()) s += ":";
+       s += name.getLocal();
+     }
    return s;
 }
+*/
 
 eFlag Processor::addKey(Sit S, const EQName& ename,
 			Expression& match, Expression &use)
@@ -1178,18 +1221,18 @@ void Processor::report(Sit S, MsgType type, MsgCode code,
 
 void Processor::initForSXP(Tree *baseTree)
 {
-    cdelete(vars);
-    input = NULL;
-    styleSheet = baseTree;
+  input = NULL;
+  styleSheet = baseTree;
+  runsOnExternal = TRUE;
+  if (!vars)
     vars = new VarsList(*styleSheet);
-    runsOnExternal = TRUE;
 }
 
 void Processor::cleanupAfterSXP()
 {
-    cdelete(vars);
-    styleSheet = NULL;
-    runsOnExternal = FALSE;
+  vars -> freeall(FALSE);
+  styleSheet = NULL;
+  runsOnExternal = FALSE;
 }
 
 Bool Processor::supportsFunction(Str &uri, Str &name) 
@@ -1266,54 +1309,78 @@ Str Processor::getNextNSPrefix()
 
 eFlag Processor::resolveGlobal(Sit S, Context *c, QName &name, XSLElement *varElem /* = NULL */)    
 { 
-    // if !varElem, we need to ask styleSheet's toplevel var directory
-    if (!varElem)
-	varElem = styleSheet -> findVarInDirectory(name);
-    else
+  // if !varElem, we need to ask styleSheet's toplevel var directory
+  /*
+  if (!varElem)
+    varElem = styleSheet -> findVarInDirectory(name);
+  else
     {
-	// set the variable name from the element
-	E( varElem -> setLogical(S, name, NZ(varElem -> atts.find(XSLA_NAME)) -> cont, FALSE) );
-}
-    if (varElem)
+      // set the variable name from the element
+      E( varElem -> setLogical(S, name, NZ(varElem -> atts.find(XSLA_NAME)) -> cont, FALSE) );
+    }
+  */
+  
+  //we have either the name or the element, if both, the element wins
+  if (varElem) 
+      E( varElem->setLogical(S, name, 
+			     NZ(varElem->atts.find(XSLA_NAME))->cont, FALSE) );
+  //this element must be the same as the varElem if available
+  XSLElement *aux = styleSheet -> findVarInDirectory(name);
+  if (varElem)
     {
-	// find the var record for several references
-	VarBindings* record = vars -> find(name);
-	// maybe it has a binding defined from a forward reference?
-	if (record && vars -> getBinding(record))
-	    return OK;
-	else
+      //it should happen only if the caller is XSLElement::execute
+      //and we have some overriden (import) declaration
+      //in such a case we simply skip the processing
+      if (varElem != aux) return OK;
+    }
+  else
+    { 
+      varElem = aux;
+    }
+
+  if (varElem)
+    {
+      // find the var record for several references
+      VarBindings* record = vars -> find(name);
+      // maybe it has a binding defined from a forward reference?
+      //but it shouldn't happen now, due the ** condition
+      if (record && vars -> getBinding(record))
+	return OK;
+      else
 	{
-	    // maybe it is an "open toplevel" (currently processed)
-	    if (record && vars -> isOpenGlobal(record))
+	  // maybe it is an "open toplevel" (currently processed)
+	  if (record && vars -> isOpenGlobal(record))
 	    {
-		Str fullName;
-		styleSheet -> expandQStr(name, fullName);
-		Err1(S, E1_VAR_CIRCULAR_REF, fullName);
+	      Str fullName;
+	      styleSheet -> expandQStr(name, fullName);
+	      Err1(S, E1_VAR_CIRCULAR_REF, fullName);
 	    }
-	    else
+	  else
 	    {
-		// process it normally
-		E( vars -> openGlobal(S, name, record) );
-		// varElem's expression will automatically evaluate with "resolvingGlobal"
-		E( varElem -> execute(S, c, /* resolvingGlobals = */ TRUE) );
-		E( vars -> closeGlobal(S, record) );		
+	      // process it normally
+	      E( vars -> openGlobal(S, name, record) );
+	      // varElem's expression will automatically evaluate with "resolvingGlobal"
+	      E( varElem -> execute(S, c, /* resolvingGlobals = */ TRUE) );
+	      E( vars -> closeGlobal(S, record) );		
 	    }
 	}
     }
-    else
+  else
     {
-	// this is not a global
-	Str fullName;
-	styleSheet -> expandQStr(name, fullName);
-	Err1(S, E1_VAR_NOT_FOUND, fullName);
+      // this is not a global
+      Str fullName;
+      styleSheet -> expandQStr(name, fullName);
+      Err1(S, E1_VAR_NOT_FOUND, fullName);
     }
-    return OK;
+  return OK;
 }
 
+/*
 eFlag Processor::resolveGlobals(Sit S, Context *c)
 {
      return styleSheet -> resolveGlobals(S, c, this);
 }
+*/
 
 eFlag Processor::stripElement(Sit S, Daddy *e)
 {
