@@ -134,6 +134,11 @@ int SablotSetOptions(SablotSituation S, int flags)
     return OK;
 }
 
+int SablotGetOptions(SablotSituation S)
+{
+  return ((Situation*)S) -> getFlags();
+}
+
 int SablotClearSituation(SablotSituation S)
 {
     ((Situation*)S) -> clear();
@@ -148,10 +153,10 @@ int SablotCreateDocument(SablotSituation S, SDOM_Document *D)
 {
     // non-XSL tree with empty name
     Tree *t = new Tree((char*)"", FALSE);
-    Arena &ar = t -> getArena();
+    SabArena &ar = t -> getArena();
     NmSpace *nm = new (&ar) NmSpace(*t, t -> unexpand("xml"), 
 				    t -> unexpand(theXMLNamespace),
-				    NSKIND_DECLARED, TRUE);
+				    TRUE, NSKIND_DECLARED);
     RootNode &root = t -> getRoot();
     root.namespaces.append(nm);
     *D = &root;
@@ -180,7 +185,7 @@ int SablotParse_(
         my_getcwd(base);
         base = Str("file://") + base;
         // should call Processor::findBaseURI() here (but have no Processor)
-        makeAbsoluteURI(uri, base, absolute);
+        makeAbsoluteURI(SIT(S), uri, base, absolute);
 	parserBase = (char*)absolute;
     }
     else
@@ -550,9 +555,11 @@ int SablotGetResultArg(void *processor_,
     int resultNdx_;
     if (argValue)
     {
-        P(processor_) -> copyArg(argURI, &resultNdx_, newCopy);
-        // if output does not go to a buffer, newCopy is NULL
-        *argValue = newCopy;
+      Situation *s = P(processor_) -> recallSituation();
+      assert(s); //we always should have some situation, master or internal
+      P(processor_) -> copyArg(SIT(s), argURI, &resultNdx_, newCopy);
+      // if output does not go to a buffer, newCopy is NULL
+      *argValue = newCopy;
     }
     return OK;
 }
