@@ -2004,6 +2004,54 @@ SDOM_Exception SDOM_xql(SablotSituation s, const SDOM_char *query, SDOM_Node cur
     return SDOM_OK;
 }
 
+SDOM_Exception SDOM_xql_ns(SablotSituation s, const SDOM_char *query, SDOM_Node currentNode, char** nsmap, SDOM_NodeList *pResult)
+{
+  //RootNode &root = toV(currentNode) -> getOwner().getRoot();
+    
+  // find the document element
+  //int i;
+  //for (i = 0; i < root.contents.number(); i++)
+  //    if (isElement(root.contents[i])) break;
+  GP( Tree ) tree = new Tree(Str("noscheme:dummy-tree"), false);
+  QName q; 
+  q.setLocal((*tree).unexpand(Str("dummy-root")));
+  Element *e = new(&((*tree).getArena())) Element(*tree, q);
+  e -> setSubtreeInfo((*tree).getRootSubtree());
+  char ** aux = nsmap;
+  while (*aux) {
+    char *pref = *aux;
+    char *uri = *(aux + 1);
+    NmSpace *nm = new(&((*tree).getArena())) 
+      NmSpace(*tree, (*tree).unexpand(pref), (*tree).unexpand(uri),
+	      false, NSKIND_DECLARED);
+    e -> newChild(SIT(s), nm);
+    aux += 2;
+  }
+	
+  Expression queryEx(*e);
+  *pResult = NULL;
+  
+  // parse the query string as a non-pattern
+  if (queryEx.parse(SIT(s), (char*)query, FALSE, TRUE))
+    SDOM_Err(s, SDOM_QUERY_PARSE_ERR);
+  
+  // create an initial context
+  GP( Context ) dummy = new Context(NULL); //_cn_ no current node
+  GP( Context ) c;
+  c.assign(dummy);
+  (*c).set(toV(currentNode));
+  
+  // create the result context
+  if (queryEx.createContext(SIT(s), c))
+    SDOM_Err(s, SDOM_QUERY_EXECUTION_ERR);
+  
+  // preserve the CList
+  (*c).getArrayForDOM() -> incRefCount();
+  c.unkeep();
+  *pResult = (SDOM_NodeList) (*c).getArrayForDOM();
+  return SDOM_OK;
+}
+
 //
 //    exception retrieval
 //
@@ -2071,4 +2119,25 @@ void SDOM_tmpListDump(SDOM_Document doc, int p)
 }
 //_TH_ ^
 
+SDOM_Exception SDOM_compareNodes(SablotSituation s, SDOM_Node n1, SDOM_Node n2, int *res)
+{
+  if (&(toV(n1) -> getOwner()) == &(toV(n2) -> getOwner()) ) //the same owner
+    {
+      int s1 = toV(n1) -> stamp;
+      int s2 = toV(n2) -> stamp;
+      if (s1 < s2)
+	*res = -1;
+      else if (s1 == s2)
+	*res = 0;
+      else 
+	*res = 1;
+    }
+  else
+    {
+      *res = strcmp( (char*)(toV(n1) -> getOwner().getURI()),
+		     (char*)(toV(n2) -> getOwner().getURI()) );
+    }
+  return SDOM_OK;
+}
+
 #endif /* ENABLE_DOM */
