@@ -126,6 +126,11 @@ void Situation::setFlags(int f)
     flags = f;
 }
 
+int Situation::getFlags()
+{
+  return flags;
+}
+
 Str Situation::timeStr()
 {
     time_t currtime;
@@ -262,7 +267,7 @@ void safeFormat(char* dest, int size, const char* format,
     {
       if (l[i] > size / __numargs)
 	{
-	  b[i] = new char[size / __numargs];
+	  b[i] = new char[size / __numargs + 1];
 	  strcpy(b[i], "...");
 	  strcpy(b[i] + 3, a[i] + (l[i] - size / __numargs + 3));
 	}
@@ -451,6 +456,10 @@ void Situation::message(MsgType type, MsgCode code,
         (!proc || !proc -> getMessageHandler(NULL)))
         generateMessage(MT_LOG, code, arg1, arg2, temp);
 
+#ifdef SABLOT_ABORT_ON_ERROR
+    if (type == MT_ERROR) abort();
+#endif
+
 }
 
 void Situation::report(Situation *sit, MsgType type, MsgCode code, const Str &str1, const Str &str2)
@@ -511,7 +520,7 @@ void Situation::swapProcessor(void *& proc_)
 const Str& Situation::findBaseURI(const Str& unmappedBase)
 {
     if (proc)
-      return proc -> findBaseURI(unmappedBase);
+      return proc -> findBaseURI(*this, unmappedBase);
     else
       return unmappedBase;
 }
