@@ -502,7 +502,8 @@ int AttSetList::findNdx(const QName &what) const
 //
 //
 
-void AliasItem::set(Phrase newKey, Phrase newValue, int newPrecedence, XSLElement *source)
+void AliasItem::set(Phrase newKey, Phrase newValue, Phrase newPrefix,
+		    int newPrecedence, XSLElement *source)
 {
     assert(newPrecedence >= 0);
     if (key == UNDEF_PHRASE) key = newKey;
@@ -512,6 +513,7 @@ void AliasItem::set(Phrase newKey, Phrase newValue, int newPrecedence, XSLElemen
     {
 	value = newValue;
 	precedence = newPrecedence;
+	prefix = newPrefix;
     }
     if (newPrecedence < precedence)
 	redefinition = NULL;
@@ -536,8 +538,8 @@ Phrase AliasList::find(Phrase key_) const
     return (ndx != -1) ? (*this)[ndx] -> getValue() : PHRASE_NOT_FOUND;
 }
 
-void AliasList::insertAlias(Phrase key, Phrase value, int precedence, 
-			    XSLElement *source)
+void AliasList::insertAlias(Phrase key, Phrase value, Phrase prefix,
+			    int precedence, XSLElement *source)
 {
     AliasItem *newItem;
     int ndx = findNdx(key);
@@ -545,7 +547,7 @@ void AliasList::insertAlias(Phrase key, Phrase value, int precedence,
 	append(newItem = new AliasItem);
     else
 	newItem = (*this)[ndx];
-    newItem -> set(key, value, precedence, source);
+    newItem -> set(key, value, prefix, precedence, source);
 }
 
 eFlag AliasList::checkRedefinitions(Sit S, Tree &sheet)
@@ -592,6 +594,7 @@ Tree::Tree(const Str &aname, BOOL aXSLTree)
 
     stripped = aXSLTree;
     hasAnyStripped = hasAnyPreserved = -1;// means 'who knows?'
+    importUnique = 0xFFFF;
 };
 
 Tree::~Tree()
@@ -710,7 +713,7 @@ HashTable& Tree::dict()
     return theDictionary;
 }
 
-Arena& Tree::getArena()
+SabArena& Tree::getArena()
 {
     return theArena;
 }
@@ -851,6 +854,17 @@ void Tree::makeStamps()
     vcount = stamp_;
 }
 
+void Tree::updateImportStatus() 
+{
+  if (! subtrees.getCurrent() -> getStructure() -> getTopLevelFound() )
+    {
+      StylesheetStructure * stru = subtrees.getCurrent() -> getStructure();
+      stru -> setTopLevelFound(true);
+      stru -> setImportPrecedence(importUnique);
+      importUnique--;
+    }
+}
+
 /*****************************************************************
 Tree::processVertexAfterParse()
 Performs any necessary actions on a vertex immediately after it is
@@ -880,8 +894,11 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
   if (isXSLElement(v))
     {
       XSLElement *x = toX(v);
+      theOp = x -> op;
 
-      switch(theOp = x -> op)
+      if (theOp != XSL_IMPORT) updateImportStatus();
+
+      switch(theOp)
         {
 	  //catch xsl:use-attribute-sets
 	case XSL_ELEMENT:
@@ -892,23 +909,11 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	  }; break;
 	case XSL_IMPORT:
 	  {
-	    // first check for ill-positioned xsl:import,
-	    // then continue as for include
-	    if (v -> parent)
+	    if (subtrees.getCurrent() -> getStructure() -> getTopLevelFound())
 	      {
-		Vertex *w;
-		int pos = v -> ordinal - 1;
-		for (; pos >= 0; pos--)
-		  {
-		    w = toD(v -> parent) -> contents[pos];
-		    if (isElement(w) && isXSL(w) &&
-			w->getImportPrecedence() <= x->getImportPrecedence())
-		      {
-			Err2(S, E_ELEM_CONTAINS_ELEM, 
-			     xslOpNames[XSL_STYLESHEET], 
-			     xslOpNames[XSL_IMPORT]);
-		      }
-		  }
+		Err2(S, E_ELEM_CONTAINS_ELEM, 
+		     xslOpNames[XSL_STYLESHEET], 
+		     xslOpNames[XSL_IMPORT]);
 	      }
 	  }; // no break
 	case XSL_INCLUDE:
@@ -919,7 +924,7 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 					    getBaseURI());		
 	    
 	    Str absolute;
-	    makeAbsoluteURI(a -> cont, base, absolute);
+	    makeAbsoluteURI(S, a -> cont, base, absolute);
 	    if (S.getProcessor())
 	      {
 		E( S.getProcessor() -> readTreeFromURI(S, srcTree, 
@@ -950,6 +955,8 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	    E( startSubtree(S, (*srcTree).getURI(), theOp) );
 	    //set extension namespaces for subtree
 
+	    //(*srcTree).speakDebug();
+
 	    //merge it into the current tree
 	    E( tc -> parseUsingSAXForAWhile(S, source, absolute, 
 					    TRUE, 
@@ -974,7 +981,7 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	      E( theSheet -> contents.copy(S, source) );
 	    E( tc -> parseUsingSAXForAWhileDone(S, source, TRUE) );
 	    // end the subtree
-	    E( endSubtree(S) );		
+	    E( endSubtree(S, theOp) );		
 	  }; break;
 	case XSL_OUTPUT:
 	  {
@@ -1034,14 +1041,29 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	  }; break;
 	case XSL_NAMESPACE_ALIAS:
 	  {
-	    Phrase style,result;
+	    Phrase style, result, sUri, rUri;
 	    Attribute *sp = NZ( x -> atts.find(XSLA_STYLESHEET_PREFIX) );
 	    Attribute *rp = NZ( x -> atts.find(XSLA_RESULT_PREFIX) );
 	    if (sp -> cont == "#default") style = UNDEF_PHRASE;
 	    else dict().insert(sp -> cont, style);
 	    if (rp -> cont == "#default") result = UNDEF_PHRASE;
 	    else dict().insert(rp -> cont, result);
-	    aliases().insertAlias(style, result, v -> getImportPrecedence(), x);
+
+	    int i;
+	    i = pendingNS().findNdx(style);
+	    if (i != -1)
+	      sUri = toNS(pendingNS().operator[](i)) -> uri;
+	    else
+	      Err1(S, E_EX_NAMESPACE_UNKNOWN, (char*) sp -> cont);
+
+	    i = pendingNS().findNdx(result);
+	    if (i != -1)
+	      rUri = toNS(pendingNS().operator[](i)) -> uri;
+	    else
+	      Err1(S, E_EX_NAMESPACE_UNKNOWN, (char*) rp -> cont);
+
+	    aliases().insertAlias(sUri, rUri, result, 
+				  v -> getImportPrecedence(), x);
 	    popVertex();
 	  }; break;
 	case XSL_TEMPLATE:
@@ -1123,7 +1145,9 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	}
 	//the literal output element may have some xsl features
     }
-    else {
+  else 
+    { //isXSLElement
+      updateImportStatus();
       if (XSLTree) {
 	E( extractUsedSets(S, toE(v)) );
 	popVertex();
@@ -1132,7 +1156,7 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
 	popVertex();
       }
     }
-    return OK;
+  return OK;
 }
 
 eFlag Tree::getSpaceNames(Sit S, Element &e, Str &str, SpaceNameList &where)
@@ -1456,12 +1480,14 @@ eFlag Tree::startSubtree(Sit S, const Str& baseURI, XSL_OP dependency,
     else
 	// find structure of current subtree (always defined since Tree::Tree)
 	structure = NZ(subtrees.getCurrent()) -> getStructure();
+
     subtrees.push(
 	new SubtreeInfo(
 	    baseURI, 
 	    dependency,
 	    structure,
 	    isInline));
+
     excludeStdNamespaces();
     //set parent tree (closest non-inline)
     if (isInline)
@@ -1479,11 +1505,19 @@ eFlag Tree::startSubtree(Sit S, const Str& baseURI, XSL_OP dependency,
     return OK;
 }
 
-eFlag Tree::endSubtree(Sit S)
+eFlag Tree::endSubtree(Sit S, XSL_OP dependency)
 {
-    // move current subtree
-    subtrees.pop();
-    return OK;
+  /*
+  subtrees.getCurrent()->getStructure()->setImportPrecedence(importUnique);
+  if (dependency == XSL_IMPORT)
+    {
+      importUnique--;
+    }
+  */
+
+  // move current subtree
+  subtrees.pop();
+  return OK;
 }
 
 StylesheetStructure* Tree::createStylesheetStructure(Sit S)
@@ -1493,8 +1527,10 @@ StylesheetStructure* Tree::createStylesheetStructure(Sit S)
     // there is always a structure for an existing subtree
     StylesheetStructure *currStruct  = NZ( currSubtree -> getStructure() );
     // create new structure with precedence one higher
-    StylesheetStructure *newStruct = new StylesheetStructure(
-	currStruct -> getImportPrecedence() + 1);
+    //StylesheetStructure *newStruct = new StylesheetStructure(
+    //currStruct -> getImportPrecedence() + 1);
+    StylesheetStructure *newStruct = new StylesheetStructure(0);
+
     // file newStruct into existing tree if stylesheet structures
     currStruct -> addImportStructure(S, newStruct);
     return newStruct;    
@@ -1505,9 +1541,11 @@ eFlag Tree::findBestRule(
     XSLElement *&ret, 
     Context *c,
     QName *currMode,
-    Bool importsOnly)
+    Bool importsOnly,
+    SubtreeInfo *subtree /* NULL */)
 {
-    return NZ(subtrees[0]) -> getStructure() -> findBestRule(
+  SubtreeInfo *start = (importsOnly && subtree) ? subtree : subtrees[0];
+    return NZ(start) -> getStructure() -> findBestRule(
 	S, ret, c, currMode, importsOnly);
 }
 
@@ -1542,6 +1580,7 @@ XSLElement* Tree::findRuleByName(QName &q)
     return NZ( subtrees[0] ) -> getStructure() -> findRuleByName(*this, q);
 }
 
+/*
 eFlag Tree::resolveGlobals(Sit S, Context *c, Processor *proc)
 {
     assert(proc);
@@ -1555,7 +1594,7 @@ eFlag Tree::resolveGlobals(Sit S, Context *c, Processor *proc)
     }
     return OK;
 }
-
+*/
 
 void Tree::setUnparsedEntityUri(Str &name, Str &uri)
 {
@@ -1581,6 +1620,15 @@ eFlag Tree::popPendingNS(Sit S)
   return OK;
 }
 
+/*
+void Tree::speakDebug()
+{
+  DStr foo;
+  getRoot().speak(foo, (SpeakMode)(SM_OFFICIAL | SM_INS_SPACES));
+  printf("--------------------\n%s\n--------------------\n", (char*)foo);
+}
+*/
+
 //_TH_ v
 /****************************************
 T m p L i s t   methods
