@@ -264,7 +264,7 @@ class StylesheetStructure
 {
 public:
     StylesheetStructure(int importPrecedence_)
-	: importPrecedence(importPrecedence_) {};
+	: importPrecedence(importPrecedence_), topLevelFound(false) {};
     ~StylesheetStructure()
 	{
 	    importChildren.freeall(FALSE);
@@ -307,11 +307,16 @@ public:
 
     // add structure for imported stylesheet
     void addImportStructure(Sit S, StylesheetStructure *imported)
-	{ importChildren.append(imported); }
+      //{ importChildren.append(imported); }
+      { importChildren.insertBefore(imported, 0); }
 
     // get the import precedence associated with this imported subtree
     int getImportPrecedence()
-	{ return importPrecedence; }
+      { return importPrecedence; }
+
+    //set the import precedence
+    void setImportPrecedence(int prec)
+      {	importPrecedence = prec; }
 
     SpaceNameList &strippedNames() { return strippedNamesList; }
     SpaceNameList &preservedNames() { return preservedNamesList; }
@@ -319,12 +324,15 @@ public:
     Bool findPreservedName(EQName &ename, int &prec, double &prio);
     Bool hasAnyStripped();
     Bool hasAnyPreserved();
+    Bool getTopLevelFound() { return topLevelFound; };
+    void setTopLevelFound(Bool val) { topLevelFound = val; }
 private:
     PList<StylesheetStructure*> importChildren;
     RuleSList rulesList;
     SpaceNameList strippedNamesList;
     SpaceNameList preservedNamesList;
     int importPrecedence;
+    Bool topLevelFound; //used during the xslt parse only, thread safe
     void report(Sit S, MsgType type, MsgCode code, 
 		const Str &arg1, const Str &arg2) const
 	{ S.message(type, code, arg1, arg2); }
@@ -415,21 +423,22 @@ class AliasItem
 {
 public:
     AliasItem()
-	:key(UNDEF_PHRASE), value(UNDEF_PHRASE),  
+      :key(UNDEF_PHRASE), value(UNDEF_PHRASE), prefix(UNDEF_PHRASE), 
       precedence(-1), redefinition(NULL) {};
-    Phrase getKey()
-	{return key;}
-    Phrase getValue()
-	{return value;}
+    Phrase getKey() {return key;}
+    Phrase getValue() {return value;}
+    Phrase getPrefix() {return prefix;}
+      
     // see if there was multiple definition with highest import precedence
     XSLElement* getRedefinition()
 	{return redefinition;}
     // sets to a new value if newPrecedence is smaller (stronger)
     // sets 'redefinition' if precedences were equal (typically an error)
     // but sets key/value anyway
-    void set(Phrase newKey, Phrase newValue, int newPrecedence, XSLElement *source);
+    void set(Phrase newKey, Phrase newValue, Phrase newPrefix,
+	     int newPrecedence, XSLElement *source);
 private:
-    Phrase key, value;
+    Phrase key, value, prefix;
     int precedence;
     XSLElement *redefinition;
 };
@@ -448,7 +457,7 @@ public:
     // find the value of the element with the given key
     Phrase find(Phrase) const;
     // insert an alias (or overwrite old if precedence is not weaker)
-    void insertAlias(Phrase key, Phrase value, int precedence, 
+    void insertAlias(Phrase key, Phrase value, Phrase prefix, int precedence, 
 		     XSLElement *source);
     eFlag checkRedefinitions(Sit S, Tree &sheet);
 private:
@@ -490,7 +499,7 @@ public:
     void flushPendingText();
     Element& dummyElement() const;
     HashTable& dict();
-    Arena& getArena();
+    SabArena& getArena();
     const QName& getTheEmptyQName() const;
     Phrase stdPhrase(StandardPhrase phrase_) const 
     { 
@@ -514,7 +523,8 @@ public:
     // finds the best matching rule
     // see StylesheetStructure::findBestRule for description
     eFlag findBestRule(Sit S, XSLElement *&ret, Context *c,
-		       QName *currMode, Bool importsOnly);
+		       QName *currMode, Bool importsOnly, 
+		       SubtreeInfo *subtree = NULL);
 
     // finds the rule of given name with highest import precedence
     XSLElement* findRuleByName(QName &q);
@@ -564,7 +574,7 @@ public:
     eFlag startSubtree(Sit S, const Str& baseURI, XSL_OP dependency,
 		       Bool isInline = FALSE);
     // end a subtree
-    eFlag endSubtree(Sit S);
+    eFlag endSubtree(Sit S, XSL_OP dependency);
     // get the tree's URI
     const Str& getURI()
       {
@@ -572,7 +582,7 @@ public:
       }
     // resolve all global variables before starting execute
     // called from Processor::resolveGlobals
-    eFlag resolveGlobals(Sit S, Context *c, Processor *proc);
+    //eFlag resolveGlobals(Sit S, Context *c, Processor *proc);
     //unparsed entties storge
     void setUnparsedEntityUri(Str &name, Str &uri);
     Str* getUnparsedEntityUri(Str &name);
@@ -584,13 +594,16 @@ public:
     Element* findStylesheet(Daddy& d);
     eFlag pushPendingNS(Sit S, Tree* srcTree, NSList &other);
     eFlag popPendingNS(Sit S);
+    SubtreeInfo* getRootSubtree() { return subtrees[0]; }
     SubtreeInfo* getCurrentInfo() 
       {return subtrees.getCurrent()->getMasterSubtree();}
     eFlag markNamespacePrefixes(Sit S);
     eFlag pushNamespacePrefixes(Sit, Str&, XSL_ATT);
     eFlag popNamespacePrefixes(Sit);
+    void updateImportStatus();
+    //void speakDebug();
 private:
-    Arena theArena;
+    SabArena theArena;
     Text *pendingTextNode;
     DStr pendingText;
     int vcount;             // # of vertices in tree
@@ -625,6 +638,7 @@ private:
     AliasList aliasesList;
     VarDirectory toplevelVars;
     int hasAnyStripped, hasAnyPreserved;
+    int importUnique;
 };
 
 #endif //ifndef TreeHIncl
