@@ -133,9 +133,9 @@ schemeToURI_()
   simply returned.
 *****************************************************************/
 
-URIScheme schemeToURI_(Str& scheme)
+URIScheme schemeToURI_(Sit S, Str& scheme)
 {
-    if (scheme.eqNoCase("file"))
+    if (scheme.eqNoCase("file") && !S.hasFlag(SAB_FILES_TO_HANDLER))
         return URI_FILE;
     else
     {
@@ -245,7 +245,8 @@ void joinPaths(Str& relPath, const Str& basePath)
 }
 
 
-URIScheme makeAbsoluteURI2(const char* uri ,const char* base, Str& absolute, Str& scheme)
+URIScheme makeAbsoluteURI2(Sit S, const char* uri,
+			   const char* base, Str& absolute, Str& scheme)
 {
     FiveStr 
         u_parts, 
@@ -290,7 +291,7 @@ URIScheme makeAbsoluteURI2(const char* uri ,const char* base, Str& absolute, Str
     DStr joined = absolute;
     joinURI(joined, u_parts, FALSE);         // join all components into a URI for return (no scheme)
     absolute = joined;
-    return schemeToURI_(scheme = u_parts[U_SCHEME]);
+    return schemeToURI_(S, scheme = u_parts[U_SCHEME]);
 }
 
 
@@ -299,17 +300,18 @@ URIScheme makeAbsoluteURI2(const char* uri ,const char* base, Str& absolute, Str
 //    Merges a (possibly relative) URI reference with a base URI, setting
 //    'absolute' to the result. 
 //
-URIScheme makeAbsoluteURI(const char* uri ,const char* base, Str& absolute)
+URIScheme makeAbsoluteURI(Sit S, const char* uri,
+			  const char* base, Str& absolute)
 {
     Str scheme;
     URIScheme temp;
-    temp = makeAbsoluteURI2(uri, base, absolute, scheme);
+    temp = makeAbsoluteURI2(S, uri, base, absolute, scheme);
     absolute = (scheme + ":") + absolute;
     return temp;
 }
 
 
-URIScheme uri2SchemePath(const char *absolute, Str& scheme, Str& rest)
+URIScheme uri2SchemePath(Sit S, const char *absolute, Str& scheme, Str& rest)
 {
     Bool found = (Bool) !!splitBy(absolute, ":", scheme);
     assert(found);
@@ -320,7 +322,7 @@ URIScheme uri2SchemePath(const char *absolute, Str& scheme, Str& rest)
  *   else
  *       rest = (char*) absolute;
  */
-    return schemeToURI_(scheme);
+    return schemeToURI_(S, scheme);
 }
 
 
@@ -411,7 +413,7 @@ eFlag DataLine::open(Sit S, const char *_uri, DLAccessMode _mode,
     assert(mode == DLMODE_NONE);  // the buffer must not be open yet
     // combine _uri and _baseUri into one
     Str strScheme, strPath;
-    scheme = uri2SchemePath(_uri, strScheme, strPath);
+    scheme = uri2SchemePath(S, _uri, strScheme, strPath);
     char *name = (char*) strPath;
 
     // mode set in the end
@@ -507,8 +509,8 @@ eFlag DataLine::close(Sit S)
             assert(f);
             if (!fileIsStd)
             {
-                if (fclose(f))
-                    Err1(S, E1_URI_CLOSE, fullUri);
+	      if (fclose(f))
+		Err1(S, E1_URI_CLOSE, fullUri);
             };
             f = NULL;
         }; break;
