@@ -795,20 +795,29 @@ eFlag Element::execute(Sit S, Context *c, Bool resolvingGlobals)
 
     EQName expandedName;
     owner.expandQ(name, expandedName);
+    OutputterObj * out = NZ(S.getProcessor()) -> outputter();
+
+    //name aliasing
+    Bool aliased = false;
+    NZ(S.getProcessor()) -> getAliasedName(expandedName, aliased);
 
     //set subtree for exclusions
-
-    E( NZ(S.getProcessor()) -> outputter() -> eventElementStart(S, 
-        expandedName) );
-
-    E( namespaces.execute(S, c, resolvingGlobals) );
+    E( out -> eventElementStart(S, expandedName) );
+    //output namespaces (and extra aliased namespace if needed)
+    E( namespaces.executeSkip(S, c, resolvingGlobals, expandedName, aliased) );
+    if (aliased)
+      E( out -> eventNamespace(S, expandedName.getPrefix(), 
+			       expandedName.getUri() ) )
+    //attribute sets
     if (attSetNames(FALSE))
       E( executeAttributeSets(S, c, resolvingGlobals) );
+    //plain attributes
     E( atts.execute(S, c, resolvingGlobals) );
+    //content
     E( Daddy::execute(S, c, resolvingGlobals) );
     removeBindings(S);
-    E( NZ(S.getProcessor()) -> outputter() -> eventElementEnd(S, 
-        expandedName));
+    //end tag
+    E( out -> eventElementEnd(S, expandedName));
 
     return OK;
 }
@@ -1297,7 +1306,7 @@ Attribute* AttList::find(const QName& attName)
 *****************************************************************/
 //NmSpace::NmSpace(Tree& owner_, Phrase _prefix, Phrase _uri, NsKind _kind)
 NmSpace::NmSpace(Tree& owner_, Phrase _prefix, Phrase _uri, 
-		 NsKind _kind, Bool _excluded)
+		 Bool _excluded, NsKind _kind)
 : Vertex(owner_, VT_NAMESPACE)
 {
     prefix = _prefix;
@@ -1315,13 +1324,7 @@ NmSpace::~NmSpace()
 eFlag NmSpace::execute(Sit S, Context *c, Bool resolvingGlobals)
 {
     assert(parent);
-/*
-    made unnecessary by the shadowing inside OutputterObj:
-    if (isXSLElement(parent) || 
-        uri == proc -> stdPhrase(PHRASE_XSL_NAMESPACE) || 
-        uri == proc -> stdPhrase(PHRASE_XML_NAMESPACE))
-        return OK;
-*/
+
     E( NZ(S.getProcessor()) -> outputter() -> 
        eventNamespace(S,
 		      //was: proc -> aliasXL(prefix)),
@@ -1332,6 +1335,20 @@ eFlag NmSpace::execute(Sit S, Context *c, Bool resolvingGlobals)
     return OK;
 }
 
+eFlag NmSpace::executeSkip(Sit S, Context *c, Bool resolvingGlobals,
+			   EQName & exName, Bool aliased)
+{
+    assert(parent);
+
+    const Str & pStr = getOwner().dict().getKey(prefix);
+    const Str & uStr = getOwner().dict().getKey(uri);
+
+    if (!aliased || !(exName.getPrefix() == pStr))
+      E( NZ(S.getProcessor()) -> outputter() -> 
+	 eventNamespace(S, pStr, uStr, excluded) );
+    return OK;
+}
+
 void NmSpace::speak(DStr& s, SpeakMode mode)
 {
     s += "xmlns";
@@ -1396,9 +1413,9 @@ NmSpace* NSList::find(Phrase prefix) const
 int NSList::findNdx(Phrase prefix) const
 {
     int i;
-    for (i = 0; i < number(); i++)
-        if (toNS((*this)[i]) -> prefix == prefix)
-           return i;
+    for (i = 0; i < number(); i++) 
+      if (toNS((*this)[i]) -> prefix == prefix)
+	return i;
     return -1;
 }
 
@@ -1450,8 +1467,8 @@ void NSList::giveCurrent(Sit S, NSList &other, Tree *t, int nscount) const
         if (other.findNdx(currNS -> prefix) == -1) {
 	    addedNS = new(&(NZ(t) -> getArena())) 
 	      NmSpace(*t, currNS -> prefix, currNS -> uri, 
-		      i > (imax - nscount) ?  NSKIND_DECLARED : NSKIND_PARENT,
-		      excludedNS.findNdx(currNS -> uri) != -1);
+		      excludedNS.findNdx(currNS->uri)!=-1 || currNS->excluded,
+		      i > (imax - nscount) ?  NSKIND_DECLARED : NSKIND_PARENT);
             t -> appendVertex(S, toV(addedNS));
 	    // _JP_ ??? why not other.append(toV(addedNS));
 	};
@@ -1535,6 +1552,16 @@ void NSList::decPrefixUsage(Phrase prefix_) const
     };
 }
 
+eFlag NSList::executeSkip(Sit S, Context * c, Bool resolvingGlobals,
+			  EQName & exName, Bool aliased)
+{
+  int i;
+  for (i=0; i<number(); i++)
+    E( toNS((*this)[i]) -> executeSkip(S, c, resolvingGlobals,
+				 exName, aliased) );
+  return OK;
+}
+
 /****************************************
 T e x t
 ****************************************/
@@ -1802,7 +1829,7 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 
     case XSL_APPLY_IMPORTS:
     {
-	E( proc -> execApplyImports(S, c, resolvingGlobals) );
+	E( proc -> execApplyImports(S, c, subtree, resolvingGlobals) );
     }; break;
     
     case XSL_IF:
@@ -1836,7 +1863,7 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 	      done = boolexpr.tobool();
 
 	      if (done) 
-		x -> execute(S, c, resolvingGlobals);
+		E( x -> execute(S, c, resolvingGlobals) );
 	    }
 	    else
 	    {
@@ -2010,7 +2037,7 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 		    //_PH_ globals
 		    QName temp;
 		    temp.empty();
-		    proc -> resolveGlobal(S, c, temp, toX(son));
+		    E( proc -> resolveGlobal(S, c, temp, toX(son)) );
 		  }; break;
 		default: E( son -> execute(S, c, resolvingGlobals) );
 		}
@@ -2049,7 +2076,7 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
     {
 	a = NZ( atts.find(XSLA_SELECT) );
 	Expression temp(*this);
-	E( NZ( a -> expr ) -> eval(S, temp, c) );
+	E( NZ( a -> expr ) -> eval(S, temp, c, resolvingGlobals) );
 	if (temp.type == EX_NODESET)
 	{
 	    const Context& ctxt = temp.tonodesetRef();
@@ -2137,7 +2164,8 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 	{
 	    proc -> vars -> startNested();
 	    TreeConstructer *newTC;
-	    E( proc -> pushTreeConstructer(S, newTC, (*expr).setFragment()) );
+	    E( proc -> pushTreeConstructer(S, newTC, (*expr).setFragment(), 
+					   SAXOUTPUT_INT_PHYSICAL) );
 	    E( contents.execute(S, c, resolvingGlobals) );
 	    E( proc -> outputter() -> eventEndOutput(S) );
 	    E( proc -> popTreeConstructer(S, newTC) );
@@ -2187,7 +2215,7 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 	// otherwise, construct a new expression; it may be an empty string
 	// if this element has void content, or a result tree fragment otherwise 
 	else
-	{
+	  {
 	    expr = new Expression(*this);
 	    if (contents.isEmpty())
 		(*expr).setAtom((const char*)"");
@@ -2202,7 +2230,8 @@ eFlag XSLElement::execute(Sit S, Context *c, Bool resolvingGlobals)
 	      if (!resolvingGlobals)
 		proc -> vars -> startNested();
 	      TreeConstructer *newTC;
-	      E( proc -> pushTreeConstructer(S, newTC, (*expr).setFragment()) );
+	      E( proc -> pushTreeConstructer(S, newTC, (*expr).setFragment(),
+					     SAXOUTPUT_INT_PHYSICAL) );
 	      // execute the inside to create the fragment
 	      E( contents.execute(S, c, resolvingGlobals) );
 	      E( proc -> outputter() -> eventEndOutput(S) );
@@ -2752,7 +2781,7 @@ eFlag ExtensionElement::executeEXSLTScript(Sit S, Context *c,
       
       const Str &base = S.findBaseURI(getSubtreeInfo() -> getBaseURI());
       Str absolute;
-      makeAbsoluteURI(a -> cont, base, absolute);
+      makeAbsoluteURI(S, a -> cont, base, absolute);
       
       eFlag status = src.open(S, absolute, DLMODE_READ,  &args);
       
@@ -2899,7 +2928,6 @@ eFlag ExtensionElement::executeEXSLTDocument(Sit S, Context *c,
   //we do not need to keep the definition, since document
   //initialized the outputter with it (I hope)
   
-
   Bool handleOutputter = !proc -> isInBinding();
   
   if (handleOutputter)
